# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Optional, TypeVar, Union, cast

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._operation_results_operations import build_get_request

if sys.version_info >= (3, 8):
    from typing import Literal  # pylint: disable=no-name-in-module, ungrouped-imports
else:
    from typing_extensions import Literal  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class OperationResultsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.botservice.aio.AzureBotService`'s
        :attr:`operation_results` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    async def _get_initial(
        self, operation_result_id: str, **kwargs: Any
    ) -> Optional[_models.OperationResultsDescription]:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-09-15"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[Optional[_models.OperationResultsDescription]] = kwargs.pop("cls", None)

        request = build_get_request(
            operation_result_id=operation_result_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self._get_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.Error, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = None
        if response.status_code == 200:
            deserialized = self._deserialize("OperationResultsDescription", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    _get_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.BotService/operationresults/{operationResultId}"
    }

    @distributed_trace_async
    async def begin_get(
        self, operation_result_id: str, **kwargs: Any
    ) -> AsyncLROPoller[_models.OperationResultsDescription]:
        """Get the operation result for a long running operation.

        :param operation_result_id: The ID of the operation result to get. Required.
        :type operation_result_id: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either OperationResultsDescription or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.botservice.models.OperationResultsDescription]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: Literal["2022-09-15"] = kwargs.pop(
            "api_version", _params.pop("api-version", self._config.api_version)
        )
        cls: ClsType[_models.OperationResultsDescription] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._get_initial(
                operation_result_id=operation_result_id,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("OperationResultsDescription", pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized

        if polling is True:
            polling_method: AsyncPollingMethod = cast(AsyncPollingMethod, AsyncARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_get.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.BotService/operationresults/{operationResultId}"
    }



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import jsondiff
import re
from enum import Enum
from typing import Any, Dict, List, Union
from _models import ChangesChecker, Suppression, RegexSuppression, PostProcessingChecker


class BreakingChangeType(str, Enum):
    REMOVED_OR_RENAMED_CLIENT = "RemovedOrRenamedClient"
    REMOVED_OR_RENAMED_CLIENT_METHOD = "RemovedOrRenamedClientMethod"
    REMOVED_OR_RENAMED_CLASS = "RemovedOrRenamedClass"
    REMOVED_OR_RENAMED_CLASS_METHOD = "RemovedOrRenamedClassMethod"
    REMOVED_OR_RENAMED_MODULE_LEVEL_FUNCTION = "RemovedOrRenamedModuleLevelFunction"
    REMOVED_OR_RENAMED_POSITIONAL_PARAM = "RemovedOrRenamedPositionalParam"
    ADDED_POSITIONAL_PARAM = "AddedPositionalParam"
    REMOVED_PARAMETER_DEFAULT_VALUE = "RemovedParameterDefaultValue"
    REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE = "RemovedOrRenamedInstanceAttribute"
    REMOVED_OR_RENAMED_ENUM_VALUE = "RemovedOrRenamedEnumValue"
    CHANGED_PARAMETER_DEFAULT_VALUE = "ChangedParameterDefaultValue"
    CHANGED_PARAMETER_ORDERING = "ChangedParameterOrdering"
    CHANGED_PARAMETER_KIND = "ChangedParameterKind"
    CHANGED_FUNCTION_KIND = "ChangedFunctionKind"
    REMOVED_OR_RENAMED_MODULE = "RemovedOrRenamedModule"
    REMOVED_FUNCTION_KWARGS = "RemovedFunctionKwargs"
    REMOVED_OR_RENAMED_OPERATION_GROUP = "RemovedOrRenamedOperationGroup"


class BreakingChangesTracker:
    REMOVED_OR_RENAMED_CLIENT_MSG = \
        "Deleted or renamed client `{}`"
    REMOVED_OR_RENAMED_CLIENT_METHOD_MSG = \
        "Deleted or renamed client method `{}.{}`"
    REMOVED_OR_RENAMED_CLASS_MSG = \
        "Deleted or renamed model `{}`"
    REMOVED_OR_RENAMED_CLASS_METHOD_MSG = \
        "Deleted or renamed method `{}.{}`"
    REMOVED_OR_RENAMED_MODULE_LEVEL_FUNCTION_MSG = \
        "Deleted or renamed function `{}`"
    REMOVED_OR_RENAMED_POSITIONAL_PARAM_OF_METHOD_MSG = \
        "Method `{}.{}` deleted or renamed its parameter `{}` of kind `{}`"
    REMOVED_OR_RENAMED_POSITIONAL_PARAM_OF_FUNCTION_MSG = \
        "Function `{}` deleted or renamed its parameter `{}` of kind `{}`"
    ADDED_POSITIONAL_PARAM_TO_METHOD_MSG = \
        "Method `{}.{}` inserted a `{}` parameter `{}`"
    ADDED_POSITIONAL_PARAM_TO_FUNCTION_MSG = \
        "Function `{}` inserted a `{}` parameter `{}`"
    REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE_FROM_CLIENT_MSG = \
        "Client `{}` deleted or renamed instance variable `{}`"
    REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE_FROM_MODEL_MSG = \
        "Model `{}` deleted or renamed its instance variable `{}`"
    REMOVED_OR_RENAMED_ENUM_VALUE_MSG = \
        "Deleted or renamed enum value `{}.{}`"
    CHANGED_PARAMETER_DEFAULT_VALUE_MSG = \
        "Method `{}.{}` parameter `{}` changed default value from `{}` to `{}`"
    CHANGED_PARAMETER_DEFAULT_VALUE_OF_FUNCTION_MSG = \
        "Function `{}` parameter `{}` changed default value from `{}` to `{}`"
    REMOVED_PARAMETER_DEFAULT_VALUE_MSG = \
        "Method `{}.{}` removed default value `{}` from its parameter `{}`"
    REMOVED_PARAMETER_DEFAULT_VALUE_OF_FUNCTION_MSG = \
        "Function `{}` removed default value `{}` from its parameter `{}`"
    CHANGED_PARAMETER_ORDERING_MSG = \
        "Method `{}.{}` re-ordered its parameters from `{}` to `{}`"
    CHANGED_PARAMETER_ORDERING_OF_FUNCTION_MSG = \
        "Function `{}` re-ordered its parameters from `{}` to `{}`"
    CHANGED_PARAMETER_KIND_MSG = \
        "Method `{}.{}` changed its parameter `{}` from `{}` to `{}`"
    CHANGED_PARAMETER_KIND_OF_FUNCTION_MSG = \
        "Function `{}` changed its parameter `{}` from `{}` to `{}`"
    CHANGED_CLASS_FUNCTION_KIND_MSG = \
        "Method `{}.{}` changed from `{}` to `{}`"
    CHANGED_FUNCTION_KIND_MSG = \
        "Changed function `{}` from `{}` to `{}`"
    REMOVED_OR_RENAMED_MODULE_MSG = \
        "Deleted or renamed module `{}`"
    REMOVED_CLASS_FUNCTION_KWARGS_MSG = \
        "Method `{}.{}` changed from accepting keyword arguments to not accepting them"
    REMOVED_FUNCTION_KWARGS_MSG = \
        "Function `{}` changed from accepting keyword arguments to not accepting them"
    REMOVED_OR_RENAMED_OPERATION_GROUP_MSG = \
        "Deleted or renamed client operation group `{}.{}`"

    def __init__(self, stable: Dict, current: Dict, package_name: str, **kwargs: Any) -> None:
        self.stable = stable
        self.current = current
        self.diff = jsondiff.diff(stable, current)
        self.features_added: List = []
        self.breaking_changes: List = []
        self.package_name = package_name
        self._module_name = None
        self._class_name = None
        self._function_name = None
        self._parameter_name = None
        self.ignore = kwargs.get("ignore", {})
        checkers: List[ChangesChecker] = kwargs.get("checkers", [])
        for checker in checkers:
            if not isinstance(checker, ChangesChecker):
                raise TypeError(f"Checker {checker} does not implement ChangesChecker protocol")
        post_processing_checkers: List[PostProcessingChecker] = kwargs.get("post_processing_checkers", [])
        for checker in post_processing_checkers:
            if not isinstance(checker, PostProcessingChecker):
                raise TypeError(f"Checker {checker} does not implement PostProcessingChecker protocol")
        self.checkers = checkers
        self.post_processing_checkers = post_processing_checkers

    def run_checks(self) -> None:
        self.run_breaking_change_diff_checks()
        self.check_parameter_ordering()  # not part of diff
        self.run_post_processing()

    def run_post_processing(self) -> None:
        # Remove duplicate reporting of changes that apply to both sync and async package components
        self.run_async_cleanup(self.breaking_changes)
        # Run user-defined post-processing checks
        for checker in self.post_processing_checkers:
            bc_list, fa_list = checker.run_check(
                self.breaking_changes, self.features_added,
                diff=self.diff, stable_nodes=self.stable, current_nodes=self.current
            )
            self.breaking_changes = bc_list
            self.features_added = fa_list
        
    # Remove duplicate reporting of changes that apply to both sync and async package components
    def run_async_cleanup(self, changes_list: List) -> None:
        def _make_hashable(item):
            """Convert an item to a hashable type.

            Lists become tuples, dicts become frozensets of items.
            Other types are returned as-is (assumed hashable).
            """
            if isinstance(item, list):
                return tuple(_make_hashable(i) for i in item)
            if isinstance(item, dict):
                return frozenset((_make_hashable(k), _make_hashable(v)) for k, v in item.items())
            return item

        def _make_key(bc):
            return tuple(_make_hashable(x) for x in (bc[1],) + bc[3:])

        # Build a set of keys from non-aio changes for O(1) lookup
        non_aio_keys = set()
        for bc in changes_list:
            if "aio" not in bc[2]:
                non_aio_keys.add(_make_key(bc))
        # Keep only non-aio changes and aio changes that don't have a sync counterpart
        changes_list[:] = [
            bc for bc in changes_list
            if "aio" not in bc[2] or _make_key(bc) not in non_aio_keys
        ]

    def run_breaking_change_diff_checks(self) -> None:
        for module_name, module in self.diff.items():
            self._module_name = module_name
            if self._module_name not in self.stable and not isinstance(self._module_name, jsondiff.Symbol):
                continue  # this is a new module/additive change in current version so skip checks

            module_deleted = self.check_module_removed_or_renamed(module)
            if module_deleted:
                continue  # module was deleted, abort other checks

            self.run_class_level_diff_checks(module)
            self.run_function_level_diff_checks(module)
        # Run custom checkers in the base class, we only need one CodeReporter class in the tool
        # The changelog reporter class is a result of not having pluggable checks, we're migrating away from it as we add more pluggable checks
        for checker in self.checkers:
            changes_list = self.breaking_changes
            if not checker.is_breaking:
                changes_list = self.features_added

            if checker.node_type == "module":
                # If we are running a module checker, we need to run it on the entire diff
                changes_list.extend(checker.run_check(self.diff, self.stable, self.current))
                continue
            if checker.node_type == "class":
                # If we are running a class checker, we need to run it on all classes in each module in the diff
                for module_name, module_components in self.diff.items():
                    # If the module_name is a symbol, we'll skip it since we can't run class checks on it
                    if not isinstance(module_name, jsondiff.Symbol):
                        changes_list.extend(checker.run_check(module_components.get("class_nodes", {}), self.stable, self.current, module_name=module_name))
                        continue
            if checker.node_type == "function_or_method":
                # If we are running a function or method checker, we need to run it on all functions and class methods in each module in the diff
                for module_name, module_components in self.diff.items():
                    # If the module_name is a symbol, we'll skip it since we can't run class checks on it
                    if not isinstance(module_name, jsondiff.Symbol):
                        for class_name, class_components in module_components.get("class_nodes", {}).items():
                            # If the class_name is a symbol, we'll skip it since we can't run method checks on it
                            if not isinstance(class_name, jsondiff.Symbol):
                                changes_list.extend(checker.run_check(class_components.get("methods", {}), self.stable, self.current, module_name=module_name, class_name=class_name))
                                continue
                        changes_list.extend(checker.run_check(module_components.get("function_nodes", {}), self.stable, self.current, module_name=module_name))


    def run_class_level_diff_checks(self, module: Dict) -> None:
        for class_name, class_components in module.get("class_nodes", {}).items():
            self._class_name = class_name
            stable_class_nodes = self.stable[self._module_name]["class_nodes"]
            if self._class_name not in stable_class_nodes and not isinstance(class_name, jsondiff.Symbol):
                continue  # this is a new model/additive change in current version so skip checks

            class_deleted = self.check_class_removed_or_renamed(class_components)
            if class_deleted:
                continue  # class was deleted, abort other checks
            self.check_class_instance_attribute_removed_or_renamed(class_components)

            for method_name, method_components in class_components.get("methods", {}).items():
                self._function_name = method_name
                stable_methods_node = stable_class_nodes[self._class_name]["methods"]
                current_methods_node = self.current[self._module_name]["class_nodes"][self._class_name]["methods"]
                if self._function_name not in stable_methods_node and \
                        not isinstance(self._function_name, jsondiff.Symbol):
                    continue  # this is a new method/additive change in current version so skip checks

                method_deleted = self.check_class_method_removed_or_renamed(method_components, stable_methods_node)
                if method_deleted:
                    continue  # method was deleted, abort other checks

                self.check_function_type_changed(method_components)

                stable_parameters_node = stable_methods_node[self._function_name]["parameters"]
                current_parameters_node = current_methods_node[self._function_name]["parameters"]
                self.run_parameter_level_diff_checks(
                    method_components,
                    stable_parameters_node,
                    current_parameters_node,
                )

    def run_function_level_diff_checks(self, module: Dict) -> None:
        self._class_name = None
        for function_name, function_components in module.get("function_nodes", {}).items():
            self._function_name = function_name
            stable_function_nodes = self.stable[self._module_name]["function_nodes"]
            if self._function_name not in stable_function_nodes and \
                    not isinstance(self._function_name, jsondiff.Symbol):
                continue  # this is a new function/additive change in current version so skip checks

            function_deleted = self.check_module_level_function_removed_or_renamed(function_components)
            if function_deleted:
                continue  # function was deleted, abort other checks

            self.check_function_type_changed(function_components)

            stable_parameters_node = stable_function_nodes[self._function_name]["parameters"]
            current_parameters_node = self.current[self._module_name]["function_nodes"][self._function_name]["parameters"]
            self.run_parameter_level_diff_checks(
                function_components,
                stable_parameters_node,
                current_parameters_node
            )

    def run_parameter_level_diff_checks(
        self, function_components: Dict,
        stable_parameters_node: Dict,
        current_parameters_node: Dict
    ) -> None:
        for param_name, diff in function_components.get("parameters", {}).items():
            self._parameter_name = param_name
            all_parameters_deleted = self.check_all_parameters_deleted(stable_parameters_node)
            if all_parameters_deleted:
                continue  # all params were removed, abort other checks
            for diff_type in diff:
                if isinstance(self._parameter_name, jsondiff.Symbol):
                    self.check_positional_parameter_removed_or_renamed(
                        stable_parameters_node[diff_type]["param_type"],
                        diff_type,
                        stable_parameters_node,
                    )
                    self.check_kwargs_removed(
                        stable_parameters_node[diff_type]["param_type"],
                        diff_type
                    )
                elif self._parameter_name not in stable_parameters_node:
                    self.check_positional_parameter_added(
                        current_parameters_node[param_name]
                    )
                    break
                elif diff_type == "default":
                    stable_default = stable_parameters_node[self._parameter_name]["default"]
                    self.check_parameter_default_value_changed_or_added(
                        diff[diff_type], stable_default
                    )
                    self.check_parameter_default_value_removed(
                        diff[diff_type], stable_default
                    )
                elif diff_type == "param_type":
                    self.check_parameter_type_changed(
                        diff["param_type"], stable_parameters_node
                    )

    def check_kwargs_removed(self, param_type: str, param_name: str) -> None:
        if param_type == "var_keyword" and param_name == "kwargs":
            if self._class_name:
                bc = (
                    self.REMOVED_CLASS_FUNCTION_KWARGS_MSG,
                    BreakingChangeType.REMOVED_FUNCTION_KWARGS,
                    self._module_name, self._class_name, self._function_name
                )
            else:
                bc = (
                    self.REMOVED_FUNCTION_KWARGS_MSG,
                    BreakingChangeType.REMOVED_FUNCTION_KWARGS,
                    self._module_name, self._function_name
                )
            self.breaking_changes.append(bc)

    def check_module_removed_or_renamed(self, module: Dict) -> Union[bool, None]:
        if isinstance(self._module_name, jsondiff.Symbol):
            deleted_modules = []
            if self._module_name.label == "delete":
                deleted_modules = [module]
            elif self._module_name.label == "replace":
                deleted_modules = self.stable

            for name in deleted_modules:
                bc = (
                    self.REMOVED_OR_RENAMED_MODULE_MSG,
                    BreakingChangeType.REMOVED_OR_RENAMED_MODULE,
                    name
                )
                self.breaking_changes.append(bc)
            return True

    def check_all_parameters_deleted(self, stable_parameters_node: Dict) -> Union[bool, None]:
        if isinstance(self._parameter_name, jsondiff.Symbol) and self._parameter_name.label == "replace":
            self.check_positional_parameter_removed_or_renamed(
                "positional_or_keyword",
                None,
                stable_parameters_node
            )
            return True

    def check_function_type_changed(self, function_components: Dict) -> None:
        value = function_components.get("is_async", None)
        if value is not None:
            if value is True:
                change = "asynchronous"
                original = "synchronous"
            else:
                change = "synchronous"
                original = "asynchronous"
            if self._class_name:
                self.breaking_changes.append(
                    (
                        self.CHANGED_CLASS_FUNCTION_KIND_MSG, BreakingChangeType.CHANGED_FUNCTION_KIND,
                        self._module_name, self._class_name, self._function_name, original, change
                    )
                )
            else:
                self.breaking_changes.append(
                    (
                        self.CHANGED_FUNCTION_KIND_MSG, BreakingChangeType.CHANGED_FUNCTION_KIND,
                        self._module_name, self._function_name, original, change
                    )
                )

    def check_parameter_type_changed(self, diff: Dict, stable_parameters_node: Dict) -> None:
        if self._class_name:
            self.breaking_changes.append(
                (
                    self.CHANGED_PARAMETER_KIND_MSG, BreakingChangeType.CHANGED_PARAMETER_KIND,
                    self._module_name, self._class_name, self._function_name, self._parameter_name,
                    stable_parameters_node[self._parameter_name]["param_type"], diff
                )
            )
        else:
            self.breaking_changes.append(
                (
                    self.CHANGED_PARAMETER_KIND_OF_FUNCTION_MSG, BreakingChangeType.CHANGED_PARAMETER_KIND,
                    self._module_name, self._function_name, self._parameter_name,
                    stable_parameters_node[self._parameter_name]["param_type"], diff
                )
            )

    def check_parameter_ordering(self) -> None:
        modules = self.stable.keys() & self.current.keys()

        for module in modules:
            if "class_nodes" in self.stable[module]:
                stable_cls = self.stable[module]["class_nodes"]
                current_cls = self.current[module]["class_nodes"]
                class_keys = stable_cls.keys() & current_cls.keys()
                for cls in class_keys:
                    stable_method_nodes = stable_cls[cls]["methods"]
                    current_method_nodes = current_cls[cls]["methods"]
                    method_keys = stable_method_nodes.keys() & current_method_nodes.keys()
                    for method in method_keys:
                        stable_params = stable_method_nodes[method]["parameters"].keys()
                        current_params = current_method_nodes[method]["parameters"].keys()
                        if len(stable_params) != len(current_params):
                            # a parameter was deleted and that breaking change was already reported so skip
                            continue
                        for key1, key2 in zip(stable_params, current_params):
                            if key1 != key2:
                                self.breaking_changes.append(
                                    (
                                        self.CHANGED_PARAMETER_ORDERING_MSG,
                                        BreakingChangeType.CHANGED_PARAMETER_ORDERING,
                                        module, cls, method, list(stable_params), list(current_params)
                                    )
                                )
                                break
            if "function_nodes" in self.stable[module]:
                stable_funcs = self.stable[module]["function_nodes"]
                current_funcs = self.current[module]["function_nodes"]
                func_nodes = stable_funcs.keys() & current_funcs.keys()
                for func in func_nodes:
                    stable_params = stable_funcs[func]["parameters"].keys()
                    current_params = current_funcs[func]["parameters"].keys()
                    if len(stable_params) != len(current_params):
                        # a parameter was deleted and that breaking change was already reported so skip
                        continue
                    for key1, key2 in zip(stable_params, current_params):
                        if key1 != key2:
                            self.breaking_changes.append(
                                (
                                    self.CHANGED_PARAMETER_ORDERING_OF_FUNCTION_MSG,
                                    BreakingChangeType.CHANGED_PARAMETER_ORDERING,
                                    module, func, list(stable_params), list(current_params)
                                )
                            )
                            break

    def check_parameter_default_value_removed(
        self, default: Union[str, None],
        stable_default: Union[str, None]
    ) -> None:
        if stable_default is not None and default is None:
            if self._class_name:
                self.breaking_changes.append(
                    (
                        self.REMOVED_PARAMETER_DEFAULT_VALUE_MSG,
                        BreakingChangeType.REMOVED_PARAMETER_DEFAULT_VALUE,
                        self._module_name, self._class_name, self._function_name,
                        default, self._parameter_name
                    )
                )
            else:

                self.breaking_changes.append(
                    (
                        self.REMOVED_PARAMETER_DEFAULT_VALUE_OF_FUNCTION_MSG,
                        BreakingChangeType.REMOVED_PARAMETER_DEFAULT_VALUE,
                        self._module_name, self._function_name, default, self._parameter_name
                    )
                )

    def check_parameter_default_value_changed_or_added(
        self, default: Union[str, None],
        stable_default: Union[str, None]
    ) -> None:
        if default is not None:  # a default was added in the current version
            if default != stable_default:
                if stable_default is not None:  # There is a stable default
                    if stable_default == "none":  # the case in which the stable default was None
                        stable_default = None  # set back to actual None for the message
                    if self._class_name:
                        self.breaking_changes.append(
                            (
                                self.CHANGED_PARAMETER_DEFAULT_VALUE_MSG,
                                BreakingChangeType.CHANGED_PARAMETER_DEFAULT_VALUE,
                                self._module_name, self._class_name, self._function_name,
                                self._parameter_name, stable_default, default
                            )
                        )
                    else:
                        self.breaking_changes.append(
                            (
                                self.CHANGED_PARAMETER_DEFAULT_VALUE_OF_FUNCTION_MSG,
                                BreakingChangeType.CHANGED_PARAMETER_DEFAULT_VALUE,
                                self._module_name, self._function_name,
                                self._parameter_name, stable_default, default
                            )
                        )

    def check_positional_parameter_added(self, current_parameters_node: Dict) -> None:
        if current_parameters_node["param_type"] == "positional_or_keyword" and \
                current_parameters_node["default"] != "none":
            if self._class_name:
                self.breaking_changes.append(
                    (
                        self.ADDED_POSITIONAL_PARAM_TO_METHOD_MSG, BreakingChangeType.ADDED_POSITIONAL_PARAM,
                        self._module_name, self._class_name, self._function_name,
                        current_parameters_node["param_type"], self._parameter_name
                    )
                )
            else:
                self.breaking_changes.append(
                    (
                        self.ADDED_POSITIONAL_PARAM_TO_FUNCTION_MSG, BreakingChangeType.ADDED_POSITIONAL_PARAM,
                        self._module_name, self._function_name,
                        current_parameters_node["param_type"], self._parameter_name
                    )
                )

    def check_positional_parameter_removed_or_renamed(
        self, param_type: str,
        deleted: str,
        stable_parameters_node: Dict
    ) -> None:
        if param_type != "positional_or_keyword":
            return
        deleted_params = []
        if self._parameter_name.label == "delete":
            deleted_params = [deleted]
        elif self._parameter_name.label == "replace":  # replace means all positional parameters were removed
            deleted_params = {
                param_name: details
                for param_name, details
                in stable_parameters_node.items()
                if details["param_type"] == "positional_or_keyword"
            }

        for deleted in deleted_params:
            if deleted != "self":
                if self._class_name:
                    self.breaking_changes.append(
                        (
                            self.REMOVED_OR_RENAMED_POSITIONAL_PARAM_OF_METHOD_MSG,
                            BreakingChangeType.REMOVED_OR_RENAMED_POSITIONAL_PARAM,
                            self._module_name, self._class_name, self._function_name, deleted, param_type
                        )
                    )
                else:
                    self.breaking_changes.append(
                        (
                            self.REMOVED_OR_RENAMED_POSITIONAL_PARAM_OF_FUNCTION_MSG,
                            BreakingChangeType.REMOVED_OR_RENAMED_POSITIONAL_PARAM,
                            self._module_name, self._function_name, deleted, param_type
                        )
                    )

    def check_class_instance_attribute_removed_or_renamed(self, components: Dict) -> None:
        deleted_props = []
        for prop in components.get("properties", []):
            if isinstance(prop, jsondiff.Symbol):
                if prop.label == "delete":
                    deleted_props = components["properties"][prop]
                elif prop.label == "replace":
                    deleted_props = self.stable[self._module_name]["class_nodes"][self._class_name]["properties"]

        for property in deleted_props:
            bc = None
            if self._class_name.endswith("Client"):
                property_type = self.stable[self._module_name]["class_nodes"][self._class_name]["properties"][property]["attr_type"]
                # property_type is not always a string, such as client_side_validation which is a bool, so we need to check for strings
                if property_type is not None and isinstance(property_type, str) and property_type.lower().endswith("operations"):
                        bc = (
                            self.REMOVED_OR_RENAMED_OPERATION_GROUP_MSG,
                            BreakingChangeType.REMOVED_OR_RENAMED_OPERATION_GROUP,
                            self._module_name, self._class_name, property
                        )
                else:
                    bc = (
                        self.REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE_FROM_CLIENT_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE,
                        self._module_name, self._class_name, property
                    )
            elif self.stable[self._module_name]["class_nodes"][self._class_name]["type"] == "Enum":
                if property.upper() not in self.current[self._module_name]["class_nodes"][self._class_name]["properties"] \
                    and property.lower() not in self.current[self._module_name]["class_nodes"][self._class_name]["properties"]:
                    bc = (
                        self.REMOVED_OR_RENAMED_ENUM_VALUE_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_ENUM_VALUE,
                        self._module_name, self._class_name, property
                    )
            else:
                bc = (
                    self.REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE_FROM_MODEL_MSG,
                    BreakingChangeType.REMOVED_OR_RENAMED_INSTANCE_ATTRIBUTE,
                    self._module_name, self._class_name, property
                )
            if bc:
                self.breaking_changes.append(bc)

    def check_class_removed_or_renamed(self, class_components: Dict) -> Union[bool, None]:
        if isinstance(self._class_name, jsondiff.Symbol):
            deleted_classes = []
            if self._class_name.label == "delete":
                deleted_classes = class_components
            elif self._class_name.label == "replace":
                deleted_classes = self.stable[self._module_name]["class_nodes"]

            for name in deleted_classes:
                if name.endswith("Client"):
                    bc = (
                        self.REMOVED_OR_RENAMED_CLIENT_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_CLIENT,
                        self._module_name, name
                    )
                else:
                    bc = (
                        self.REMOVED_OR_RENAMED_CLASS_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_CLASS,
                        self._module_name, name
                    )
                self.breaking_changes.append(bc)
            return True

    def check_class_method_removed_or_renamed(
        self, method_components: Dict,
        stable_methods_node: Dict
    ) -> Union[bool, None]:
        if isinstance(self._function_name, jsondiff.Symbol):
            methods_deleted = []
            if self._function_name.label == "delete":
                methods_deleted = method_components
            elif self._function_name.label == "replace":
                methods_deleted = stable_methods_node

            for method in methods_deleted:
                if self._class_name.endswith("Client"):
                    bc = (
                        self.REMOVED_OR_RENAMED_CLIENT_METHOD_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_CLIENT_METHOD,
                        self._module_name, self._class_name, method
                    )
                else:
                    bc = (
                        self.REMOVED_OR_RENAMED_CLASS_METHOD_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_CLASS_METHOD,
                        self._module_name, self._class_name, method
                    )
                self.breaking_changes.append(bc)
            return True

    def check_module_level_function_removed_or_renamed(self, function_components: Dict) -> Union[bool, None]:
        if isinstance(self._function_name, jsondiff.Symbol):
            deleted_functions = []
            if self._function_name.label == "delete":
                deleted_functions = function_components
            elif self._function_name.label == "replace":
                deleted_functions = self.stable[self._module_name]["function_nodes"]

            for function in deleted_functions:
                self.breaking_changes.append(
                    (
                        self.REMOVED_OR_RENAMED_MODULE_LEVEL_FUNCTION_MSG,
                        BreakingChangeType.REMOVED_OR_RENAMED_MODULE_LEVEL_FUNCTION,
                        self._module_name, function
                    )
                )
            return True

    def match(self, bc, ignored):
        if bc == ignored:
            return True
        for b, i in zip(bc, ignored):
            if i == "*" or i is None:
                # If the ignore rule is a wildcard or None, we skip this part of the check
                continue
            if isinstance(i, RegexSuppression) and b is not None:
                if i.match(b):
                    continue
                else:
                    return False
            if b != i:
                return False
        return True

    def get_reportable_changes(self, ignore_changes: Dict, changes_list: List) -> List:
        ignored = []
        # Match all ignore rules that should apply to this package
        for ignored_package, ignore_rules in ignore_changes.items():
            if re.search(ignored_package, self.package_name):
                ignored.extend(ignore_rules)

        if not ignored:
            return

        # Pre-create Suppression objects once instead of recreating per change
        suppressions = [Suppression(*rule) for rule in ignored]

        # Filter out ignored changes in a single pass instead of deepcopy + list.remove()
        filtered = []
        for bc in changes_list:
            _, bc_type, module_name, *args = bc
            class_name = args[0] if args else None
            function_name = args[1] if len(args) > 1 else None
            parameter_name = args[2] if len(args) > 2 else None

            should_keep = True
            for suppression in suppressions:
                if suppression.parameter_or_property_name is not None:
                    # If the ignore rule is for a property or parameter, we should check up to that level on the original change
                    if self.match((bc_type, module_name, class_name, function_name, parameter_name), suppression):
                        should_keep = False
                        break
                elif self.match((bc_type, module_name, class_name, function_name), suppression):
                    should_keep = False
                    break
            if should_keep:
                filtered.append(bc)

        changes_list[:] = filtered

    def report_changes(self) -> None:
        ignore_changes = self.ignore if self.ignore else {}
        self.get_reportable_changes(ignore_changes, self.breaking_changes)

        # If there are no breaking changes after the ignore check, return early
        if not self.breaking_changes:
            return f"\nNo breaking changes found for {self.package_name} between versions."

        formatted = "\n"
        for idx, bc in enumerate(self.breaking_changes):
            msg, bc_type, module_name, *args = bc
            if bc_type == BreakingChangeType.REMOVED_OR_RENAMED_MODULE:
                # For module-level changes, the module name is the first argument
                formatted += f"({bc_type}): " + msg.format(module_name) + "\n"
                continue
            # For simple breaking changes reporting, prepend the change code to the message
            msg = f"({bc_type}): " + msg + "\n"
            formatted += msg.format(*args)
        
        formatted += f"\nFound {len(self.breaking_changes)} breaking changes.\n"
        formatted += "\nSee aka.ms/azsdk/breaking-changes-tool to resolve " \
                     "any reported breaking changes or false positives.\n"
        return formatted



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    Address,
    AnonymousModelRequest,
    BinaryArrayPartsRequest,
    ComplexHttpPartsModelRequest,
    ComplexPartsRequest,
    FileWithHttpPartOptionalContentTypeRequest,
    FileWithHttpPartRequiredContentTypeRequest,
    FileWithHttpPartSpecificContentTypeRequest,
    FloatRequest,
    JsonPartRequest,
    MultiBinaryPartsRequest,
    MultiPartOptionalRequest,
    MultiPartRequest,
    MultiPartRequestWithWireName,
    UploadFileArrayRequest,
    UploadFileRequiredFilenameRequest,
    UploadFileSpecificContentTypeRequest,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Address",
    "AnonymousModelRequest",
    "BinaryArrayPartsRequest",
    "ComplexHttpPartsModelRequest",
    "ComplexPartsRequest",
    "FileWithHttpPartOptionalContentTypeRequest",
    "FileWithHttpPartRequiredContentTypeRequest",
    "FileWithHttpPartSpecificContentTypeRequest",
    "FloatRequest",
    "JsonPartRequest",
    "MultiBinaryPartsRequest",
    "MultiPartOptionalRequest",
    "MultiPartRequest",
    "MultiPartRequestWithWireName",
    "UploadFileArrayRequest",
    "UploadFileRequiredFilenameRequest",
    "UploadFileSpecificContentTypeRequest",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# pylint: disable=too-many-lines, C4763
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

""" Custom Async Polling Methods for Azure Batch Operations."""

import asyncio

from typing import Any, Callable, Optional

from azure.core.exceptions import AzureError, ResourceNotFoundError
from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncPollingMethod

from ... import models as _models


class DeleteJobPollingMethodAsync(AsyncPollingMethod):
    """Polling method for job delete operation.

    This class is used to poll the status of a job deletion operation.
    It checks the status of the job until it is deleted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the DeleteJobPollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the job
        deletion status.

        :param client: An instance of the Batch client used to make API calls for
            checking job status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            delete job API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param job_id: The unique identifier of the job being deleted.
        :type job_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._polling_interval = polling_interval
        self._status = "InProgress"
        self._finished = False

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = await self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.DELETING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job no longer exists, deletion is complete
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Some other error occurred, operation failed
            self._status = "Failed"
            self._finished = True


class DisableJobPollingMethodAsync(AsyncPollingMethod):
    """Polling method for job disable operation.

    This class is used to poll the status of a job disable operation.
    It checks the status of the job until it is disabled or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the DisableJobPollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the job
        disable operation status.

        :param client: An instance of the Batch client used to make API calls for
            checking job status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            disable job API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param job_id: The unique identifier of the job being disabled.
        :type job_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._polling_interval = polling_interval
        self._status = "InProgress"
        self._finished = False

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = await self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.DISABLING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # If job doesn't exist it could've been deleted while disabling
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class EnableJobPollingMethodAsync(AsyncPollingMethod):
    """Polling method for job enable operation.

    This class is used to poll the status of a job enable operation.
    It checks the status of the job until it is enabled or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the EnableJobPollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the job
        enable operation status.

        :param client: An instance of the Batch client used to make API calls for
            checking job status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            enable job API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param job_id: The unique identifier of the job being enabled.
        :type job_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._polling_interval = polling_interval
        self._status = "InProgress"
        self._finished = False

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = await self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.ENABLING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # If job doesn't exist it could've been deleted while enabling
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class DeleteJobSchedulePollingMethodAsync(AsyncPollingMethod):
    """Polling method for job schedule delete operation.

    This class is used to poll the status of a job schedule deletion operation.
    It checks the status of the job schedule until it is deleted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_schedule_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the DeleteJobSchedulePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the job
        schedule deletion status.

        :param client: An instance of the Batch client used to make API calls for
            checking job schedule status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            delete job schedule API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param job_schedule_id: The unique identifier of the job schedule being deleted.
        :type job_schedule_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_schedule_id = job_schedule_id
        self._polling_interval = polling_interval
        self._status = "InProgress"
        self._finished = False

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job_schedule = await self._client.get_job_schedule(self._job_schedule_id)

            if job_schedule.state == _models.BatchJobScheduleState.DELETING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job schedule no longer exists, deletion is complete
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class DeletePoolPollingMethodAsync(AsyncPollingMethod):
    """Polling method for pool delete operation.
    This class is used to poll the status of a pool deletion operation.
    It checks the status of the pool until it is deleted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the DeletePoolPollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the pool
        deletion status.

        :param client: An instance of the Batch client used to make API calls for
            checking pool status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            delete pool API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool being deleted.
        :type pool_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._polling_interval = polling_interval
        self._status = "InProgress"
        self._finished = False

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = await self._client.get_pool(self._pool_id)

            if pool.state == _models.BatchPoolState.DELETING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Pool no longer exists, deletion is complete
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class DeallocateNodePollingMethodAsync(AsyncPollingMethod):
    """Polling method for node deallocate operation.

    This class is used to poll the status of a node deallocation operation.
    It checks the status of the node until it is deallocated or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        """Initialize the DeallocateNodePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the node
        deallocation status.

        :param client: An instance of the Batch client used to make API calls for
            checking node status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            deallocate node API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool containing the node
            to be deallocated.
        :type pool_id: str
        :keyword node_id: The unique identifier of the node to be deallocated.
        :paramtype node_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = await self._client.get_node(self._pool_id, self._node_id)

            # If node not in DEALLOCATING state then completed
            # DEALLOCATED is too quick of a state to check for success
            if node.state == _models.BatchNodeState.DEALLOCATING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node no longer exists, might have been removed from pool
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class RebootNodePollingMethodAsync(AsyncPollingMethod):
    """Polling method for node reboot operation.

    This class is used to poll the status of a node reboot operation.
    It checks the status of the node until it is rebooted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        """Initialize the RebootNodePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the node
        reboot status.

        :param client: An instance of the Batch client used to make API calls for
            checking node status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            reboot node API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool containing the node
            to be rebooted.
        :type pool_id: str
        :keyword node_id: The unique identifier of the node to be rebooted.
        :paramtype node_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = await self._client.get_node(self._pool_id, self._node_id)

            if node.state == _models.BatchNodeState.REBOOTING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node could be deleted while rebooting
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class ReimageNodePollingMethodAsync(AsyncPollingMethod):
    """Polling method for node reimage operation.

    This class is used to poll the status of a node reimage operation.
    It checks the status of the node until it is reimaged or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        """Initialize the ReimageNodePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the node
        reimage status.

        :param client: An instance of the Batch client used to make API calls for
            checking node status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            reimage node API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool containing the node
            to be reimaged.
        :type pool_id: str
        :keyword node_id: The unique identifier of the node to be reimaged.
        :paramtype node_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = await self._client.get_node(self._pool_id, self._node_id)

            if node.state == _models.BatchNodeState.REIMAGING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node could be deleted while reimaging
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class RemoveNodePollingMethodAsync(AsyncPollingMethod):
    """Polling method for node remove operation.

    This class is used to poll the status of a node removal operation.
    It checks the status of the node until it is removed or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the RemoveNodePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the node
        removal status.

        :param client: An instance of the Batch client used to make API calls for
            checking pool status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            remove node API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool from which nodes
            are being removed.
        :type pool_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = await self._client.get_pool(self._pool_id)

            # Node removal is complete when the pool allocation state is STEADY
            # This means the pool is no longer resizing/removing nodes
            if pool.allocation_state == _models.AllocationState.STEADY:
                self._status = "Succeeded"
                self._finished = True
            else:
                self._status = "InProgress"
                self._finished = False

        except ResourceNotFoundError:
            # Pool could be deleted while removing nodes
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class ResizePoolPollingMethodAsync(AsyncPollingMethod):
    """Polling method for pool resize operation.

    This class is used to poll the status of a pool resize operation.
    It checks the status of the pool until it is resized or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the ResizePoolPollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the pool
        resize status.

        :param client: An instance of the Batch client used to make API calls for
            checking pool status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            resize pool API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool being resized.
        :type pool_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = await self._client.get_pool(self._pool_id)

            # Pool resize is complete when the pool allocation state is STEADY
            # This means the pool is no longer resizing (adding/removing nodes)
            if pool.allocation_state == _models.AllocationState.STEADY:
                self._status = "Succeeded"
                self._finished = True
            else:
                self._status = "InProgress"
                self._finished = False

        except ResourceNotFoundError:
            # Pool could be deleted while resizing
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class StartNodePollingMethodAsync(AsyncPollingMethod):
    """Polling method for node start operation.

    This class is used to poll the status of a node start operation.
    It checks the status of the node until it is started or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        """Initialize the StartNodePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the node
        start status.

        :param client: An instance of the Batch client used to make API calls for
            checking node status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            start node API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool containing the node
            to be started.
        :type pool_id: str
        :keyword node_id: The unique identifier of the node to be started.
        :paramtype node_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = await self._client.get_node(self._pool_id, self._node_id)

            if node.state == _models.BatchNodeState.STARTING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node could be deleted during starting
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class StopPoolResizePollingMethodAsync(AsyncPollingMethod):
    """Polling method for pool stop resize operation.

    This class is used to poll the status of a pool stop resize operation.
    It checks the status of the pool until it is stopped or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the StopPoolResizePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the pool
        stop resize status.

        :param client: An instance of the Batch client used to make API calls for
            checking pool status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            stop pool resize API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param pool_id: The unique identifier of the pool for which resize is
            being stopped.
        :type pool_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = await self._client.get_pool(self._pool_id)

            # Pool stop resize is complete when the pool allocation state is STEADY
            # This means the pool has stopped resizing and is stable
            if pool.allocation_state == _models.AllocationState.STEADY:
                self._status = "Succeeded"
                self._finished = True
            else:
                self._status = "InProgress"
                self._finished = False

        except ResourceNotFoundError:
            # Pool could be deleted during stop resize operation
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class TerminateJobPollingMethodAsync(AsyncPollingMethod):
    """Polling method for job termination operation.

    This class is used to poll the status of a job termination operation.
    It checks the status of the job until it is terminated or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the TerminateJobPollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the job
        termination status.

        :param client: An instance of the Batch client used to make API calls for
            checking job status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            terminate job API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param job_id: The unique identifier of the job being terminated.
        :type job_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = await self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.TERMINATING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job could be deleted while terminating
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class TerminateJobSchedulePollingMethodAsync(AsyncPollingMethod):
    """Polling method for job schedule termination operation.

    This class is used to poll the status of a job schedule termination operation.
    It checks the status of the job schedule until it is terminated or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_schedule_id: str,
        *,
        polling_interval: int = 5,
    ):
        """Initialize the TerminateJobSchedulePollingMethodAsync.

        Sets the initial status of this long-running operation, verifies that polling
        is possible, and initializes all necessary components for polling the job
        schedule termination status.

        :param client: An instance of the Batch client used to make API calls for
            checking job schedule status during polling.
        :type client: Any
        :param initial_response: The PipelineResponse returned from the initial
            terminate job schedule API call.
        :type initial_response: ~azure.core.pipeline.PipelineResponse
        :param deserialization_callback: A callable function to transform the final
            result before returning to the end user. Can be None if no transformation
            is needed.
        :type deserialization_callback: Optional[Callable]
        :param job_schedule_id: The unique identifier of the job schedule being
            terminated.
        :type job_schedule_id: str
        :keyword polling_interval: The time interval in seconds between polling
            attempts. Defaults to 5 seconds.
        :paramtype polling_interval: int
        """
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_schedule_id = job_schedule_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    async def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            await self.update_status()
            if not self.finished():
                # add a delay if not done
                await asyncio.sleep(self._polling_interval)

    async def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job_schedule = await self._client.get_job_schedule(self._job_schedule_id)

            # Job schedule termination is complete when it's no longer in TERMINATING state
            if job_schedule.state == _models.BatchJobScheduleState.TERMINATING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job schedule could be deleted while terminating
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import ActivationKeyResult
from ._models_py3 import CloudManifestFileDeploymentData
from ._models_py3 import CloudManifestFileProperties
from ._models_py3 import CloudManifestFileResponse
from ._models_py3 import Compatibility
from ._models_py3 import CustomerSubscription
from ._models_py3 import CustomerSubscriptionList
from ._models_py3 import DataDiskImage
from ._models_py3 import DeploymentLicenseRequest
from ._models_py3 import DeploymentLicenseResponse
from ._models_py3 import DeviceConfiguration
from ._models_py3 import Display
from ._models_py3 import ErrorDetails
from ._models_py3 import ErrorResponse
from ._models_py3 import ExtendedProduct
from ._models_py3 import ExtendedProductProperties
from ._models_py3 import IconUris
from ._models_py3 import MarketplaceProductLogUpdate
from ._models_py3 import Operation
from ._models_py3 import OperationList
from ._models_py3 import OsDiskImage
from ._models_py3 import Product
from ._models_py3 import ProductLink
from ._models_py3 import ProductList
from ._models_py3 import ProductLog
from ._models_py3 import ProductProperties
from ._models_py3 import Registration
from ._models_py3 import RegistrationList
from ._models_py3 import RegistrationParameter
from ._models_py3 import Resource
from ._models_py3 import TrackedResource
from ._models_py3 import VirtualMachineExtensionProductProperties
from ._models_py3 import VirtualMachineProductProperties

from ._azure_stack_management_client_enums import Category
from ._azure_stack_management_client_enums import CompatibilityIssue
from ._azure_stack_management_client_enums import ComputeRole
from ._azure_stack_management_client_enums import Location
from ._azure_stack_management_client_enums import OperatingSystem
from ._azure_stack_management_client_enums import ProvisioningState
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ActivationKeyResult",
    "CloudManifestFileDeploymentData",
    "CloudManifestFileProperties",
    "CloudManifestFileResponse",
    "Compatibility",
    "CustomerSubscription",
    "CustomerSubscriptionList",
    "DataDiskImage",
    "DeploymentLicenseRequest",
    "DeploymentLicenseResponse",
    "DeviceConfiguration",
    "Display",
    "ErrorDetails",
    "ErrorResponse",
    "ExtendedProduct",
    "ExtendedProductProperties",
    "IconUris",
    "MarketplaceProductLogUpdate",
    "Operation",
    "OperationList",
    "OsDiskImage",
    "Product",
    "ProductLink",
    "ProductList",
    "ProductLog",
    "ProductProperties",
    "Registration",
    "RegistrationList",
    "RegistrationParameter",
    "Resource",
    "TrackedResource",
    "VirtualMachineExtensionProductProperties",
    "VirtualMachineProductProperties",
    "Category",
    "CompatibilityIssue",
    "ComputeRole",
    "Location",
    "OperatingSystem",
    "ProvisioningState",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# coding=utf-8

from copy import deepcopy
from typing import Any, Awaitable
from typing_extensions import Self

from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient, policies

from .._utils.serialization import Deserializer, Serializer
from ._configuration import ValueTypesClientConfiguration
from .operations import (
    BooleanLiteralOperations,
    BooleanOperations,
    BytesOperations,
    CollectionsIntOperations,
    CollectionsModelOperations,
    CollectionsStringOperations,
    DatetimeOperations,
    Decimal128Operations,
    DecimalOperations,
    DictionaryStringOperations,
    DurationOperations,
    EnumOperations,
    ExtensibleEnumOperations,
    FloatLiteralOperations,
    FloatOperations,
    IntLiteralOperations,
    IntOperations,
    ModelOperations,
    NeverOperations,
    StringLiteralOperations,
    StringOperations,
    UnionEnumValueOperations,
    UnionFloatLiteralOperations,
    UnionIntLiteralOperations,
    UnionStringLiteralOperations,
    UnknownArrayOperations,
    UnknownDictOperations,
    UnknownIntOperations,
    UnknownStringOperations,
)


class ValueTypesClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """Illustrates various property types for models.

    :ivar boolean: BooleanOperations operations
    :vartype boolean: typetest.property.valuetypes.aio.operations.BooleanOperations
    :ivar string: StringOperations operations
    :vartype string: typetest.property.valuetypes.aio.operations.StringOperations
    :ivar bytes: BytesOperations operations
    :vartype bytes: typetest.property.valuetypes.aio.operations.BytesOperations
    :ivar int_operations: IntOperations operations
    :vartype int_operations: typetest.property.valuetypes.aio.operations.IntOperations
    :ivar float: FloatOperations operations
    :vartype float: typetest.property.valuetypes.aio.operations.FloatOperations
    :ivar decimal: DecimalOperations operations
    :vartype decimal: typetest.property.valuetypes.aio.operations.DecimalOperations
    :ivar decimal128: Decimal128Operations operations
    :vartype decimal128: typetest.property.valuetypes.aio.operations.Decimal128Operations
    :ivar datetime: DatetimeOperations operations
    :vartype datetime: typetest.property.valuetypes.aio.operations.DatetimeOperations
    :ivar duration: DurationOperations operations
    :vartype duration: typetest.property.valuetypes.aio.operations.DurationOperations
    :ivar enum: EnumOperations operations
    :vartype enum: typetest.property.valuetypes.aio.operations.EnumOperations
    :ivar extensible_enum: ExtensibleEnumOperations operations
    :vartype extensible_enum: typetest.property.valuetypes.aio.operations.ExtensibleEnumOperations
    :ivar model: ModelOperations operations
    :vartype model: typetest.property.valuetypes.aio.operations.ModelOperations
    :ivar collections_string: CollectionsStringOperations operations
    :vartype collections_string:
     typetest.property.valuetypes.aio.operations.CollectionsStringOperations
    :ivar collections_int: CollectionsIntOperations operations
    :vartype collections_int: typetest.property.valuetypes.aio.operations.CollectionsIntOperations
    :ivar collections_model: CollectionsModelOperations operations
    :vartype collections_model:
     typetest.property.valuetypes.aio.operations.CollectionsModelOperations
    :ivar dictionary_string: DictionaryStringOperations operations
    :vartype dictionary_string:
     typetest.property.valuetypes.aio.operations.DictionaryStringOperations
    :ivar never: NeverOperations operations
    :vartype never: typetest.property.valuetypes.aio.operations.NeverOperations
    :ivar unknown_string: UnknownStringOperations operations
    :vartype unknown_string: typetest.property.valuetypes.aio.operations.UnknownStringOperations
    :ivar unknown_int: UnknownIntOperations operations
    :vartype unknown_int: typetest.property.valuetypes.aio.operations.UnknownIntOperations
    :ivar unknown_dict: UnknownDictOperations operations
    :vartype unknown_dict: typetest.property.valuetypes.aio.operations.UnknownDictOperations
    :ivar unknown_array: UnknownArrayOperations operations
    :vartype unknown_array: typetest.property.valuetypes.aio.operations.UnknownArrayOperations
    :ivar string_literal: StringLiteralOperations operations
    :vartype string_literal: typetest.property.valuetypes.aio.operations.StringLiteralOperations
    :ivar int_literal: IntLiteralOperations operations
    :vartype int_literal: typetest.property.valuetypes.aio.operations.IntLiteralOperations
    :ivar float_literal: FloatLiteralOperations operations
    :vartype float_literal: typetest.property.valuetypes.aio.operations.FloatLiteralOperations
    :ivar boolean_literal: BooleanLiteralOperations operations
    :vartype boolean_literal: typetest.property.valuetypes.aio.operations.BooleanLiteralOperations
    :ivar union_string_literal: UnionStringLiteralOperations operations
    :vartype union_string_literal:
     typetest.property.valuetypes.aio.operations.UnionStringLiteralOperations
    :ivar union_int_literal: UnionIntLiteralOperations operations
    :vartype union_int_literal:
     typetest.property.valuetypes.aio.operations.UnionIntLiteralOperations
    :ivar union_float_literal: UnionFloatLiteralOperations operations
    :vartype union_float_literal:
     typetest.property.valuetypes.aio.operations.UnionFloatLiteralOperations
    :ivar union_enum_value: UnionEnumValueOperations operations
    :vartype union_enum_value: typetest.property.valuetypes.aio.operations.UnionEnumValueOperations
    :keyword endpoint: Service host. Default value is "http://localhost:3000".
    :paramtype endpoint: str
    """

    def __init__(  # pylint: disable=missing-client-constructor-parameter-credential
        self, *, endpoint: str = "http://localhost:3000", **kwargs: Any
    ) -> None:
        _endpoint = "{endpoint}"
        self._config = ValueTypesClientConfiguration(endpoint=endpoint, **kwargs)

        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.logging_policy,
            ]
        self._client: AsyncPipelineClient = AsyncPipelineClient(endpoint=_endpoint, policies=_policies, **kwargs)

        self._serialize = Serializer()
        self._deserialize = Deserializer()
        self._serialize.client_side_validation = False
        self.boolean = BooleanOperations(self._client, self._config, self._serialize, self._deserialize)
        self.string = StringOperations(self._client, self._config, self._serialize, self._deserialize)
        self.bytes = BytesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.int_operations = IntOperations(self._client, self._config, self._serialize, self._deserialize)
        self.float = FloatOperations(self._client, self._config, self._serialize, self._deserialize)
        self.decimal = DecimalOperations(self._client, self._config, self._serialize, self._deserialize)
        self.decimal128 = Decimal128Operations(self._client, self._config, self._serialize, self._deserialize)
        self.datetime = DatetimeOperations(self._client, self._config, self._serialize, self._deserialize)
        self.duration = DurationOperations(self._client, self._config, self._serialize, self._deserialize)
        self.enum = EnumOperations(self._client, self._config, self._serialize, self._deserialize)
        self.extensible_enum = ExtensibleEnumOperations(self._client, self._config, self._serialize, self._deserialize)
        self.model = ModelOperations(self._client, self._config, self._serialize, self._deserialize)
        self.collections_string = CollectionsStringOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.collections_int = CollectionsIntOperations(self._client, self._config, self._serialize, self._deserialize)
        self.collections_model = CollectionsModelOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.dictionary_string = DictionaryStringOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.never = NeverOperations(self._client, self._config, self._serialize, self._deserialize)
        self.unknown_string = UnknownStringOperations(self._client, self._config, self._serialize, self._deserialize)
        self.unknown_int = UnknownIntOperations(self._client, self._config, self._serialize, self._deserialize)
        self.unknown_dict = UnknownDictOperations(self._client, self._config, self._serialize, self._deserialize)
        self.unknown_array = UnknownArrayOperations(self._client, self._config, self._serialize, self._deserialize)
        self.string_literal = StringLiteralOperations(self._client, self._config, self._serialize, self._deserialize)
        self.int_literal = IntLiteralOperations(self._client, self._config, self._serialize, self._deserialize)
        self.float_literal = FloatLiteralOperations(self._client, self._config, self._serialize, self._deserialize)
        self.boolean_literal = BooleanLiteralOperations(self._client, self._config, self._serialize, self._deserialize)
        self.union_string_literal = UnionStringLiteralOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.union_int_literal = UnionIntLiteralOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.union_float_literal = UnionFloatLiteralOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.union_enum_value = UnionEnumValueOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import DiscountsOperations  # type: ignore
from ._operations import ContributorsOperations  # type: ignore
from ._operations import CreditsOperations  # type: ignore
from ._operations import ConditionalCreditsOperations  # type: ignore
from ._operations import ConditionalCreditContributorsOperations  # type: ignore
from ._operations import MaccsOperations  # type: ignore
from ._operations import SavingsPlanOrderAliasOperations  # type: ignore
from ._operations import SavingsPlanOrderOperations  # type: ignore
from ._operations import SavingsPlanOperations  # type: ignore
from ._operations import ReservationOrderAliasOperations  # type: ignore
from ._operations import DiscountOperations  # type: ignore
from ._operations import BenefitOperations  # type: ignore
from ._operations import SellerResourceOperations  # type: ignore
from ._operations import ApplicableMaccsOperations  # type: ignore
from ._operations import SourcesOperations  # type: ignore
from ._operations import FreeServicesOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "DiscountsOperations",
    "ContributorsOperations",
    "CreditsOperations",
    "ConditionalCreditsOperations",
    "ConditionalCreditContributorsOperations",
    "MaccsOperations",
    "SavingsPlanOrderAliasOperations",
    "SavingsPlanOrderOperations",
    "SavingsPlanOperations",
    "ReservationOrderAliasOperations",
    "DiscountOperations",
    "BenefitOperations",
    "SellerResourceOperations",
    "ApplicableMaccsOperations",
    "SourcesOperations",
    "FreeServicesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.appcontainers import ContainerAppsAPIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestContainerAppsAPIBuildsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ContainerAppsAPIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_builds_get(self, resource_group):
        response = self.client.builds.get(
            resource_group_name=resource_group.name,
            builder_name="str",
            build_name="str",
            api_version="2025-02-02-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_builds_begin_create_or_update(self, resource_group):
        response = self.client.builds.begin_create_or_update(
            resource_group_name=resource_group.name,
            builder_name="str",
            build_name="str",
            build_envelope={
                "buildStatus": "str",
                "configuration": {
                    "baseOs": "str",
                    "environmentVariables": [{"name": "str", "value": "str"}],
                    "platform": "str",
                    "platformVersion": "str",
                    "preBuildSteps": [
                        {
                            "description": "str",
                            "httpGet": {"url": "str", "fileName": "str", "headers": ["str"]},
                            "scripts": ["str"],
                        }
                    ],
                },
                "destinationContainerRegistry": {"server": "str", "image": "str"},
                "id": "str",
                "logStreamEndpoint": "str",
                "name": "str",
                "provisioningState": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tokenEndpoint": "str",
                "type": "str",
                "uploadEndpoint": "str",
            },
            api_version="2025-02-02-preview",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_builds_begin_delete(self, resource_group):
        response = self.client.builds.begin_delete(
            resource_group_name=resource_group.name,
            builder_name="str",
            build_name="str",
            api_version="2025-02-02-preview",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Tests for graceful shutdown with InvocationAgentServerHost."""
import asyncio
import logging

import pytest
from httpx import ASGITransport, AsyncClient
from starlette.requests import Request
from starlette.responses import Response

from azure.ai.agentserver.invocations import InvocationAgentServerHost



# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------

def _make_server_with_shutdown(**kwargs) -> tuple[InvocationAgentServerHost, list]:
    """Create InvocationAgentServerHost with a tracked shutdown handler."""
    server = InvocationAgentServerHost(**kwargs)
    calls: list[str] = []

    @server.invoke_handler
    async def handle(request: Request) -> Response:
        return Response(content=b"ok")

    @server.shutdown_handler
    async def on_shutdown():
        calls.append("shutdown")

    return server, calls


# ---------------------------------------------------------------------------
# Shutdown handler registration
# ---------------------------------------------------------------------------

def test_shutdown_handler_registered():
    """Shutdown handler is stored on the server."""
    server, _ = _make_server_with_shutdown()
    assert server._shutdown_fn is not None


def test_shutdown_handler_not_registered():
    """Without @shutdown_handler, _shutdown_fn is None."""
    app = InvocationAgentServerHost()

    @app.invoke_handler
    async def handle(request: Request) -> Response:
        return Response(content=b"ok")

    assert app._shutdown_fn is None


# ---------------------------------------------------------------------------
# ASGI lifespan helper
# ---------------------------------------------------------------------------

async def _drive_lifespan(app):
    """Drive a full ASGI lifespan startup+shutdown cycle."""
    scope = {"type": "lifespan"}
    startup_done = asyncio.Event()
    shutdown_done = asyncio.Event()

    async def receive():
        if not startup_done.is_set():
            startup_done.set()
            return {"type": "lifespan.startup"}
        await asyncio.sleep(0)
        return {"type": "lifespan.shutdown"}

    async def send(message):
        if message["type"] == "lifespan.shutdown.complete":
            shutdown_done.set()

    await app(scope, receive, send)
    return shutdown_done.is_set()


# ---------------------------------------------------------------------------
# Shutdown handler called during lifespan
# ---------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_shutdown_handler_called_on_lifespan_exit():
    """Shutdown handler runs when the ASGI lifespan exits."""
    server, calls = _make_server_with_shutdown()

    # Drive the lifespan via raw ASGI protocol
    completed = await _drive_lifespan(server)
    assert completed
    assert "shutdown" in calls


# ---------------------------------------------------------------------------
# Shutdown handler timeout
# ---------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_shutdown_handler_timeout(caplog):
    """Shutdown handler that exceeds timeout is warned about."""
    server = InvocationAgentServerHost(graceful_shutdown_timeout=1)
    calls: list[str] = []

    @server.invoke_handler
    async def handle(request: Request) -> Response:
        return Response(content=b"ok")

    @server.shutdown_handler
    async def on_shutdown():
        await asyncio.sleep(10)
        calls.append("completed")

    with caplog.at_level(logging.WARNING, logger="azure.ai.agentserver"):
        await _drive_lifespan(server)

    # Shutdown should have been interrupted
    assert "completed" not in calls
    # Logger should have warned about timeout
    assert any("did not complete" in r.message.lower() or "timeout" in r.message.lower() for r in caplog.records)


# ---------------------------------------------------------------------------
# Shutdown handler exception
# ---------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_shutdown_handler_exception(caplog):
    """Shutdown handler that raises is caught and logged."""
    app = InvocationAgentServerHost()

    @app.invoke_handler
    async def handle(request: Request) -> Response:
        return Response(content=b"ok")

    @app.shutdown_handler
    async def on_shutdown():
        raise RuntimeError("shutdown exploded")

    with caplog.at_level(logging.WARNING, logger="azure.ai.agentserver"):
        await _drive_lifespan(app)

    # Should have logged the exception
    assert any("on_shutdown" in r.message.lower() or "error" in r.message.lower() for r in caplog.records)


# ---------------------------------------------------------------------------
# Graceful shutdown timeout config
# ---------------------------------------------------------------------------

def test_default_graceful_shutdown_timeout():
    """Default graceful shutdown timeout is 30 seconds."""
    app = InvocationAgentServerHost()
    assert app._graceful_shutdown_timeout == 30


def test_custom_graceful_shutdown_timeout():
    """Custom graceful_shutdown_timeout is stored."""
    server = InvocationAgentServerHost(graceful_shutdown_timeout=60)
    assert server._graceful_shutdown_timeout == 60


def test_zero_graceful_shutdown_timeout():
    """Zero timeout disables the drain period."""
    server = InvocationAgentServerHost(graceful_shutdown_timeout=0)
    assert server._graceful_shutdown_timeout == 0


# ---------------------------------------------------------------------------
# Health endpoint accessible during normal operation
# ---------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_health_endpoint_during_operation():
    """GET /readiness returns 200 during normal operation."""
    server, _ = _make_server_with_shutdown()
    transport = ASGITransport(app=server)
    async with AsyncClient(transport=transport, base_url="http://testserver") as client:
        resp = await client.get("/readiness")
    assert resp.status_code == 200
    assert resp.json() == {"status": "healthy"}


# ---------------------------------------------------------------------------
# No shutdown handler is no-op
# ---------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_no_shutdown_handler_is_noop():
    """Without a shutdown handler, lifespan exit succeeds silently."""
    app = InvocationAgentServerHost()

    @app.invoke_handler
    async def handle(request: Request) -> Response:
        return Response(content=b"ok")

    transport = ASGITransport(app=app)
    async with AsyncClient(transport=transport, base_url="http://testserver") as client:
        resp = await client.post("/invocations", content=b"test")
        assert resp.status_code == 200
    # No exception means success


# ---------------------------------------------------------------------------
# Multiple requests before shutdown
# ---------------------------------------------------------------------------

@pytest.mark.asyncio
async def test_multiple_requests_before_shutdown():
    """Multiple requests can be served, then shutdown handler runs."""
    server, calls = _make_server_with_shutdown()
    transport = ASGITransport(app=server)
    async with AsyncClient(transport=transport, base_url="http://testserver") as client:
        for i in range(5):
            resp = await client.post("/invocations", content=f"request-{i}".encode())
            assert resp.status_code == 200

    # Drive the lifespan to trigger shutdown
    completed = await _drive_lifespan(server)
    assert completed
    assert "shutdown" in calls



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to use basic agent operations from
    the Azure Agents service using a synchronous client with Azure Monitor tracing.
    View the results in the "Tracing" tab in your Azure AI Foundry project page.

USAGE:
    python sample_agents_basics_with_azure_monitor_tracing.py

    Before running the sample:

    pip install azure-ai-projects azure-ai-agents azure-identity azure-monitor-opentelemetry

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
       page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
    3) OTEL_INSTRUMENTATION_GENAI_CAPTURE_MESSAGE_CONTENT - Optional. Set to `true` to trace the content of chat
       messages, which may contain personal data. False by default.
    4) APPLICATIONINSIGHTS_CONNECTION_STRING - Set to the connection string of your Application Insights resource.
       This is used to send telemetry data to Azure Monitor. You can also get the connection string programmatically
       from AIProjectClient using the `telemetry.get_application_insights_connection_string()` method. A code sample showing
       how to do this can be found in the `sample_telemetry.py` file in the azure-ai-projects telemetry samples.
"""

import os
from azure.ai.projects import AIProjectClient
from azure.ai.agents.models import ListSortOrder
from azure.identity import DefaultAzureCredential

project_client = AIProjectClient(
    endpoint=os.environ["PROJECT_ENDPOINT"],
    credential=DefaultAzureCredential(),
)

# [START enable_tracing]
from opentelemetry import trace
from azure.monitor.opentelemetry import configure_azure_monitor

# Enable Azure Monitor tracing
application_insights_connection_string = os.environ["APPLICATIONINSIGHTS_CONNECTION_STRING"]
configure_azure_monitor(connection_string=application_insights_connection_string)

scenario = os.path.basename(__file__)
tracer = trace.get_tracer(__name__)

with tracer.start_as_current_span(scenario):
    with project_client:
        agents_client = project_client.agents

        # [END enable_tracing]
        agent = agents_client.create_agent(
            model=os.environ["MODEL_DEPLOYMENT_NAME"], name="my-agent", instructions="You are helpful agent"
        )
        print(f"Created agent, agent ID: {agent.id}")

        thread = agents_client.threads.create()
        print(f"Created thread, thread ID: {thread.id}")

        message = agents_client.messages.create(
            thread_id=thread.id, role="user", content="Hello, tell me a hilarious joke"
        )
        print(f"Created message, message ID: {message.id}")

        run = agents_client.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
        print(f"Run completed with status: {run.status}")

        agents_client.delete_agent(agent.id)
        print("Deleted agent")

        messages = agents_client.messages.list(thread_id=thread.id, order=ListSortOrder.ASCENDING)
        for msg in messages:
            if msg.text_messages:
                last_text = msg.text_messages[-1]
                print(f"{msg.role}: {last_text.text.value}")



from abc import ABC
from typing import Generic, TYPE_CHECKING, TypeVar

if TYPE_CHECKING:
    from .serialization import Deserializer, Serializer


TClient = TypeVar("TClient")
TConfig = TypeVar("TConfig")


class ClientMixinABC(ABC, Generic[TClient, TConfig]):
    """DO NOT use this class. It is for internal typing use only."""

    _client: TClient
    _config: TConfig
    _serialize: "Serializer"
    _deserialize: "Deserializer"



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to run Prompt Agent operations
    using the File Search Tool and a synchronous client.

USAGE:
    python sample_agent_file_search.py

    Before running the sample:

    pip install "azure-ai-projects>=2.0.0" python-dotenv

    Set these environment variables with your own values:
    1) FOUNDRY_PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
       page of your Microsoft Foundry portal.
    2) FOUNDRY_MODEL_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Microsoft Foundry project.
"""

import os
from dotenv import load_dotenv

# Azure AI imports
from azure.identity import DefaultAzureCredential
from azure.ai.projects import AIProjectClient
from azure.ai.projects.models import PromptAgentDefinition, FileSearchTool

load_dotenv()

endpoint = os.environ["FOUNDRY_PROJECT_ENDPOINT"]

with (
    DefaultAzureCredential() as credential,
    AIProjectClient(endpoint=endpoint, credential=credential) as project_client,
    project_client.get_openai_client() as openai_client,
):
    # [START tool_declaration]
    # Create vector store for file search
    vector_store = openai_client.vector_stores.create(name="ProductInfoStore")
    print(f"Vector store created (id: {vector_store.id})")

    # Load the file to be indexed for search
    asset_file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../assets/product_info.md"))

    # Upload file to vector store
    with open(asset_file_path, "rb") as f:
        file = openai_client.vector_stores.files.upload_and_poll(vector_store_id=vector_store.id, file=f)
    print(f"File uploaded to vector store (id: {file.id})")

    tool = FileSearchTool(vector_store_ids=[vector_store.id])
    # [END tool_declaration]

    # Create agent with file search tool
    agent = project_client.agents.create_version(
        agent_name="MyAgent",
        definition=PromptAgentDefinition(
            model=os.environ["FOUNDRY_MODEL_NAME"],
            instructions="You are a helpful assistant that can search through product information.",
            tools=[tool],
        ),
        description="File search agent for product information queries.",
    )
    print(f"Agent created (id: {agent.id}, name: {agent.name}, version: {agent.version})")

    # Create a conversation for the agent interaction
    conversation = openai_client.conversations.create()
    print(f"Created conversation (id: {conversation.id})")

    # Send a query to search through the uploaded file
    response = openai_client.responses.create(
        conversation=conversation.id,
        input="Tell me about Contoso products",
        extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
    )
    print(f"Agent response: {response.output_text}")
    print("\nCleaning up...")
    project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)
    print("Agent deleted")



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import unittest
from unittest.mock import patch, MagicMock
import datetime
from azure.appconfiguration.provider._constants import (
    DEFAULT_STARTUP_TIMEOUT,
    MAX_STARTUP_BACKOFF_DURATION,
    MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION,
    STARTUP_BACKOFF_INTERVALS,
    JITTER_RATIO,
)


class TestStartupRetryIntegration(unittest.TestCase):
    """Integration tests for startup retry behavior."""

    def test_default_startup_timeout_value(self):
        """Test that DEFAULT_STARTUP_TIMEOUT has expected value."""
        self.assertEqual(DEFAULT_STARTUP_TIMEOUT, 100)

    def test_max_backoff_duration_value(self):
        """Test that MAX_STARTUP_BACKOFF_DURATION has expected value."""
        self.assertEqual(MAX_STARTUP_BACKOFF_DURATION, 600)

    def test_MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION_value(self):
        """Test that MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION has expected value."""
        self.assertEqual(MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION, 30)

    def test_startup_backoff_intervals_structure(self):
        """Test that STARTUP_BACKOFF_INTERVALS has expected structure."""
        self.assertEqual(len(STARTUP_BACKOFF_INTERVALS), 3)
        # Should be sorted by threshold
        thresholds = [interval[0] for interval in STARTUP_BACKOFF_INTERVALS]
        self.assertEqual(thresholds, sorted(thresholds))

    def test_jitter_ratio_value(self):
        """Test that JITTER_RATIO has expected value."""
        self.assertEqual(JITTER_RATIO, 0.25)


class TestLoadAllRetryBehavior(unittest.TestCase):
    """Test the _load_all method retry behavior."""

    def setUp(self):
        """Set up test fixtures."""
        # We'll use mocking to test the retry behavior

    @patch("azure.appconfiguration.provider._azureappconfigurationprovider.datetime")
    def test_load_all_timeout_raises_timeout_error(self, mock_datetime):
        """Test that _load_all raises TimeoutError when startup_timeout is exceeded."""
        from azure.appconfiguration.provider._azureappconfigurationprovider import AzureAppConfigurationProvider
        from azure.appconfiguration.provider._azureappconfigurationproviderbase import AzureAppConfigurationProviderBase

        # Create a mock provider - use a custom __init__ that calls base but doesn't set up client manager
        def mock_init(self, **kwargs):
            AzureAppConfigurationProviderBase.__init__(self, endpoint="http://test.endpoint", **kwargs)
            self._startup_timeout = 1  # 1 second timeout
            self._replica_client_manager = MagicMock()
            self._secret_provider = MagicMock()
            self._secret_provider.uses_key_vault = False

        with patch.object(AzureAppConfigurationProvider, "__init__", mock_init):
            provider = AzureAppConfigurationProvider()
            provider._replica_client_manager.get_next_active_client.return_value = None

            # Mock datetime to simulate time passing
            start_time = datetime.datetime(2026, 1, 1, 0, 0, 0)
            times = [
                start_time,  # Initial startup_start_time
                start_time,  # First loop elapsed check (0 seconds elapsed)
                start_time,  # First loop remaining timeout check
                start_time + datetime.timedelta(seconds=2),  # Second loop check exceeds timeout
            ]
            mock_datetime.datetime.now.side_effect = times

            with self.assertRaises(TimeoutError) as context:
                provider._load_all()

            self.assertIn("timed out", str(context.exception.args[0]).lower())

    @patch("azure.appconfiguration.provider._azureappconfigurationprovider.time.sleep")
    def test_load_all_uses_fixed_backoff_initially(self, mock_sleep):
        """Test that _load_all uses fixed backoff during initial period."""
        from azure.appconfiguration.provider._azureappconfigurationprovider import AzureAppConfigurationProvider
        from azure.appconfiguration.provider._azureappconfigurationproviderbase import AzureAppConfigurationProviderBase

        # Create a mock provider - use a custom __init__ that calls base but doesn't set up client manager
        def mock_init(self, **kwargs):
            AzureAppConfigurationProviderBase.__init__(self, endpoint="http://test.endpoint", **kwargs)
            self._startup_timeout = 10
            self._replica_client_manager = MagicMock()
            self._secret_provider = MagicMock()
            self._secret_provider.uses_key_vault = False

        with patch.object(AzureAppConfigurationProvider, "__init__", mock_init):
            provider = AzureAppConfigurationProvider()

            # First attempt fails, second succeeds
            attempt_count = [0]

            def mock_try_initialize(startup_exceptions):
                attempt_count[0] += 1
                if attempt_count[0] == 1:
                    startup_exceptions.append(ConnectionError("Connection failed"))
                    return False
                return True

            provider._try_initialize = mock_try_initialize

            provider._load_all()

            # Verify sleep was called with fixed backoff duration (5 seconds for first 100 seconds)
            mock_sleep.assert_called_once()
            sleep_duration = mock_sleep.call_args[0][0]
            self.assertEqual(sleep_duration, 5)

    @patch("azure.appconfiguration.provider._azureappconfigurationprovider.time.sleep")
    def test_load_all_succeeds_on_first_try(self, mock_sleep):
        """Test that _load_all succeeds without retries when first attempt succeeds."""
        from azure.appconfiguration.provider._azureappconfigurationprovider import AzureAppConfigurationProvider
        from azure.appconfiguration.provider._azureappconfigurationproviderbase import AzureAppConfigurationProviderBase

        # Create a mock provider - use a custom __init__ that calls base but doesn't set up client manager
        def mock_init(self, **kwargs):
            AzureAppConfigurationProviderBase.__init__(self, endpoint="http://test.endpoint", **kwargs)
            self._startup_timeout = 100
            self._replica_client_manager = MagicMock()
            self._secret_provider = MagicMock()
            self._secret_provider.uses_key_vault = False

        with patch.object(AzureAppConfigurationProvider, "__init__", mock_init):
            provider = AzureAppConfigurationProvider()

            # First attempt succeeds
            provider._try_initialize = MagicMock(return_value=True)

            provider._load_all()

            # Sleep should not be called when first attempt succeeds
            mock_sleep.assert_not_called()
            provider._try_initialize.assert_called_once()


class TestStartupTimeoutConfiguration(unittest.TestCase):
    """Test startup_timeout configuration."""

    def test_custom_startup_timeout(self):
        """Test that custom startup_timeout is respected."""
        from azure.appconfiguration.provider._azureappconfigurationprovider import _buildprovider

        with patch("azure.appconfiguration.provider._azureappconfigurationprovider.ConfigurationClientManager"):
            with patch("azure.appconfiguration.provider._azureappconfigurationprovider.SecretProvider"):
                provider = _buildprovider(
                    connection_string="Endpoint=https://fake.endpoint;Id=fake;Secret=fake",
                    endpoint=None,
                    credential=None,
                    startup_timeout=60,
                )
                self.assertEqual(provider._startup_timeout, 60)

    def test_default_startup_timeout(self):
        """Test that default startup_timeout is used when not specified."""
        from azure.appconfiguration.provider._azureappconfigurationprovider import _buildprovider

        with patch("azure.appconfiguration.provider._azureappconfigurationprovider.ConfigurationClientManager"):
            with patch("azure.appconfiguration.provider._azureappconfigurationprovider.SecretProvider"):
                provider = _buildprovider(
                    connection_string="Endpoint=https://fake.endpoint;Id=fake;Secret=fake",
                    endpoint=None,
                    credential=None,
                )
                self.assertEqual(provider._startup_timeout, DEFAULT_STARTUP_TIMEOUT)


if __name__ == "__main__":
    unittest.main()



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to perform CRUD operations on agent Sessions
    using the asynchronous AIProjectClient.

    Sessions only work with Hosted Agents.

    Sessions are currently a preview feature. In the Python SDK, you access
    these operations via `project_client.beta.agents`.

USAGE:
    python sample_sessions_crud_async.py

    Before running the sample:

    pip install "azure-ai-projects>=2.1.0" python-dotenv aiohttp

    Set these environment variables with your own values:
    1) FOUNDRY_PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
       page of your Microsoft Foundry portal.
    2) FOUNDRY_AGENT_CONTAINER_IMAGE - The Hosted Agent container image in the format '<registry>/<repository>[:<tag>|@<digest>]'

    You can build and push an example image from
    `samples/hosted_agents/assets/responses-echo-agent` and use that image value
    for `FOUNDRY_AGENT_CONTAINER_IMAGE`.
"""

import asyncio
import os

from dotenv import load_dotenv

from azure.identity.aio import DefaultAzureCredential

from azure.ai.projects.aio import AIProjectClient
from azure.ai.projects.models import HostedAgentDefinition, ProtocolVersionRecord, VersionRefIndicator
from hosted_agents_util import wait_for_agent_version_active_async

load_dotenv()

endpoint = os.environ["FOUNDRY_PROJECT_ENDPOINT"]
image = os.environ["FOUNDRY_AGENT_CONTAINER_IMAGE"]


async def main() -> None:
    async with (
        DefaultAzureCredential() as credential,
        AIProjectClient(
            endpoint=endpoint,
            credential=credential,
            allow_preview=True,
        ) as project_client,
    ):
        agent_name = "MySessionHostedAgent"

        # Create an agent version to back the session.
        agent = await project_client.agents.create_version(
            agent_name=agent_name,
            definition=HostedAgentDefinition(
                cpu="0.5",
                memory="1Gi",
                image=image,
                container_protocol_versions=[
                    ProtocolVersionRecord(protocol="responses", version="v1"),
                ],
            ),
            metadata={"enableVnextExperience": "true"},
        )
        print(f"Agent created (name: {agent.name}, version: {agent.version})")

        await wait_for_agent_version_active_async(
            project_client=project_client,
            agent_name=agent_name,
            agent_version=agent.version,
        )

        isolation_key = "sample-isolation-key"
        session = await project_client.beta.agents.create_session(
            agent_name=agent_name,
            isolation_key=isolation_key,
            version_indicator=VersionRefIndicator(agent_version=agent.version),
        )
        print(f"Session created (id: {session.agent_session_id}, status: {session.status})")

        try:
            fetched = await project_client.beta.agents.get_session(
                agent_name=agent_name,
                session_id=session.agent_session_id,
            )
            print(f"Retrieved session (id: {fetched.agent_session_id}, status: {fetched.status})")

            print("Listing sessions for the agent...")
            sessions = []
            async for item in project_client.beta.agents.list_sessions(agent_name=agent_name):
                sessions.append(item)
            print("Sessions:")
            for item in sessions:
                print(f"  - {item.agent_session_id} (status: {item.status})")
        finally:
            print(f"Deleting session with id: {session.agent_session_id}...")
            await project_client.beta.agents.delete_session(
                agent_name=agent_name,
                session_id=session.agent_session_id,
                isolation_key=isolation_key,
            )
            print(f"Session with id: {session.agent_session_id} deleted.")

            await project_client.agents.delete_version(agent_name=agent_name, agent_version=agent.version)
            print(f"Agent version {agent.version} deleted.")


if __name__ == "__main__":
    asyncio.run(main())



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# This script is used to execute pyright within a tox environment.

from subprocess import check_call, CalledProcessError
import argparse
import os
import logging
import sys
import json

from ci_tools.environment_exclusions import (
    is_check_enabled, is_typing_ignored
)
from ci_tools.parsing import ParsedSetup
from ci_tools.variables import in_ci

logging.getLogger().setLevel(logging.INFO)

root_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", "..", ".."))


def get_pyright_config_path(args):
    """Give pyright an execution environment when running with tox. Otherwise
    we use pyright's default for import resolution which doesn't work well
    in our monorepo.

    Since we don't want to be burdened with upkeep of this config for every library
    that runs pyright checks, nor do we need this when running pyright without tox,
    we'll add the config on the fly when invoked with tox.
    """

    # find the repo-level or package-level pyrightconfig.json
    user_config_path = args.target_package if os.path.exists(os.path.join(args.target_package, "pyrightconfig.json")) else root_dir

    # read the config and adjust relative paths
    with open(os.path.join(user_config_path, "pyrightconfig.json"), "r") as f:
        config_text = f.read()
        config_text = config_text.replace("\"**", "\"../../../../**")
        config = json.loads(config_text)

    # add or update the execution environment for tox
    if config.get("executionEnvironments"):
        config["executionEnvironments"].append({"root": args.target_package})
    else:
        config.update({"executionEnvironments": [{"root": args.target_package}]})

    # write the pyrightconfig.json to the tox environment and return the path so we can point to it
    pyright_env = "pyright" if not args.next else "next-pyright"
    pyright_config_path = os.path.join(args.target_package, ".tox", pyright_env, "tmp", "pyrightconfig.json")
    with open(pyright_config_path, "w+") as f:
        f.write(json.dumps(config, indent=4))
    return pyright_config_path


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run pyright against target folder. ")

    parser.add_argument(
        "-t",
        "--target",
        dest="target_package",
        help="The target package directory on disk. The target module passed to run pyright will be <target_package>/azure.",
        required=True,
    )

    parser.add_argument(
        "--next",
        default=False,
        help="Next version of pyright is being tested.",
        required=False
    )

    args = parser.parse_args()
    package_dir = os.path.abspath(args.target_package)
    package_name = os.path.basename(package_dir)

    if not args.next and in_ci():
        if not is_check_enabled(args.target_package, "pyright") or is_typing_ignored(package_name):
            logging.info(
                f"Package {package_name} opts-out of pyright check. See https://aka.ms/python/typing-guide for information."
            )
            exit(0)

    pkg_details = ParsedSetup.from_path(package_dir)
    top_level_module = pkg_details.namespace.split(".")[0]
    paths = [
        os.path.join(args.target_package, top_level_module),
    ]
    if not args.next and in_ci() and not is_check_enabled(args.target_package, "type_check_samples"):
        logging.info(
            f"Package {package_name} opts-out of pyright check on samples."
        )
    else:
        # check if samples dir exists, if not, skip sample code check
        samples = os.path.exists(os.path.join(args.target_package, "samples"))
        generated_samples = os.path.exists(os.path.join(args.target_package, "generated_samples"))
        if not samples and not generated_samples:
            logging.info(
                f"Package {package_name} does not have a samples directory."
            )
        else:
            paths.append(os.path.join(args.target_package, "samples" if samples else "generated_samples"))


    pyright_config_path = get_pyright_config_path(args)

    commands = [
        sys.executable,
        "-m",
        "pyright",
        "--project",
        pyright_config_path,
    ]
    commands.extend(paths)
    try:
        check_call(commands)
    except CalledProcessError as error:
        if args.next and in_ci() and is_check_enabled(args.target_package, "pyright") and not is_typing_ignored(package_name):
            from gh_tools.vnext_issue_creator import create_vnext_issue
            create_vnext_issue(package_dir, "pyright")

        print("See https://aka.ms/python/typing-guide for information.\n\n")
        raise error

    if args.next and in_ci() and not is_typing_ignored(package_name):
        from gh_tools.vnext_issue_creator import close_vnext_issue
        close_vnext_issue(package_name, "pyright")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import functools
import pytest

from devtools_testutils import (
    AzureRecordedTestCase,
    EnvironmentVariableLoader,
)
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.textanalytics.aio import TextAnalysisClient
from azure.ai.textanalytics.models import (
    MultiLanguageTextInput,
    MultiLanguageInput,
    CustomMultiLabelClassificationActionContent,
    CustomMultiLabelClassificationOperationAction,
    TextActions,
    CustomMultiLabelClassificationOperationResult,
    ClassificationActionResult,
    ClassificationResult,
)

TextAnalysisPreparer = functools.partial(
    EnvironmentVariableLoader,
    "text_analysis",
    text_analysis_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    text_analysis_key="fake_key",
)


class TestTextAnalysisAsync(AzureRecordedTestCase):
    def create_client(self, endpoint: str, key: str) -> TextAnalysisClient:
        return TextAnalysisClient(endpoint, AzureKeyCredential(key))


class TestTextAnalysisCaseAsync(TestTextAnalysisAsync):
    @TextAnalysisPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_multi_label_classify_async(
        self, text_analysis_endpoint, text_analysis_key
    ):
        async with self.create_client(text_analysis_endpoint, text_analysis_key) as client:
            project_name = "multi-class-project"
            deployment_name = "multiclassdeployment"

            text_a = (
                "I need a reservation for an indoor restaurant in China. Please don't stop the music. "
                "Play music and add it to my playlist."
            )

            text_input = MultiLanguageTextInput(
                multi_language_inputs=[MultiLanguageInput(id="A", text=text_a, language="en")]
            )

            # Start LRO (async)
            poller = await client.begin_analyze_text_job(
                text_input=text_input,
                actions=[
                    CustomMultiLabelClassificationOperationAction(
                        name="CMCOperationActionSample",
                        action_content=CustomMultiLabelClassificationActionContent(
                            project_name=project_name,
                            deployment_name=deployment_name,
                        ),
                    )
                ],
            )

            assert poller is not None
            paged_actions = await poller.result()
            details = poller.details
            assert "operation_id" in details
            assert details.get("status") is not None
            assert paged_actions is not None

            found_cmc = False

            async for actions_page in paged_actions:
                # Page container holding job results
                assert isinstance(actions_page, TextActions)
                assert actions_page.items_property is not None  # wire: "items"

                for op_result in actions_page.items_property:
                    if isinstance(op_result, CustomMultiLabelClassificationOperationResult):
                        found_cmc = True
                        result = op_result.results
                        assert result is not None
                        assert result.documents is not None

                        for doc in result.documents:
                            assert isinstance(doc, ClassificationActionResult)
                            assert doc.id is not None

                            assert doc.class_property is not None
                            for cls_item in doc.class_property:
                                assert isinstance(cls_item, ClassificationResult)
                                assert cls_item.category is not None
                                assert cls_item.confidence_score is not None

            assert found_cmc, "Expected a CustomMultiLabelClassificationOperationResult in TextActions.items_property"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import SpecialWordsClientTestBase, SpecialWordsPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestSpecialWordsOperations(SpecialWordsClientTestBase):
    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_and_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.and_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_as_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.as_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_assert_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.assert_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_async_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.async_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_await_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.await_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_break_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.break_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_class_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.class_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_constructor(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.constructor()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_continue_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.continue_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_def_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.def_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_del_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.del_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_elif_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.elif_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_else_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.else_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_except_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.except_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_exec_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.exec_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_finally_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.finally_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_for_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.for_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_from_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.from_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_global_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.global_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_if_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.if_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_import_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.import_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_in_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.in_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_is_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.is_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_lambda_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.lambda_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_not_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.not_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_or_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.or_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_pass_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.pass_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_raise_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.raise_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_return_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.return_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_try_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.try_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_while_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.while_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_with_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.with_method()

        # please add some check logic here by yourself
        # ...

    @SpecialWordsPreparer()
    @recorded_by_proxy
    def test_operations_yield_method(self, specialwords_endpoint):
        client = self.create_client(endpoint=specialwords_endpoint)
        response = client.operations.yield_method()

        # please add some check logic here by yourself
        # ...
